@@ -16,8 +16,6 @@
 {-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE TypeSynonymInstances       #-}
-{-# LANGUAGE UndecidableInstances       #-}
-  -- UndecidableInstances needed for ghc < 707
 
 ----------------------------------------------------------------------------
 -- |
@@ -65,7 +63,7 @@
 -- >                          --   section of the output.
 -- >    , _idPrefix        :: T.Text
 -- >    , _svgAttributes   :: [Attribute]
--- >                          -- ^ Attriubtes to apply to the entire svg element.
+-- >                          -- ^ Attributes to apply to the entire svg element.
 -- >    , _generateDoctype :: Bool
 -- >    }
 --
@@ -100,6 +98,7 @@ module Diagrams.Backend.SVG
   , B
     -- for rendering options specific to SVG
   , Options(..), sizeSpec, svgDefinitions, idPrefix, svgAttributes, generateDoctype
+  , svgClass, svgId, svgTitle
   , SVGFloat
 
   , renderSVG
@@ -110,8 +109,9 @@ module Diagrams.Backend.SVG
   ) where
 
 -- from JuicyPixels
-import           Codec.Picture
-import           Codec.Picture.Types      (dynamicMap)
+import           Codec.Picture            (decodeImage, encodeDynamicPng)
+import           Codec.Picture.Types      (DynamicImage (ImageYCbCr8),
+                                           dynamicMap, imageHeight, imageWidth)
 
 #if __GLASGOW_HASKELL__ < 710
 import           Data.Foldable            as F (foldMap)
@@ -139,10 +139,11 @@ import           Control.Lens             hiding (transform, ( # ))
 
 -- from diagrams-core
 import           Diagrams.Core.Compile
-import           Diagrams.Core.Types      (Annotation (..))
+import           Diagrams.Core.Types      (Annotation (..), keyVal)
 
 -- from diagrams-lib
-import           Diagrams.Prelude         hiding (Attribute, size, view, local)
+import           Diagrams.Prelude         hiding (Attribute, local, size, view,
+                                           with)
 import           Diagrams.TwoD.Adjust     (adjustDia2D)
 import           Diagrams.TwoD.Attributes (FillTexture, splitTextureFills)
 import           Diagrams.TwoD.Path       (Clip (Clip))
@@ -167,7 +168,7 @@ type instance N SVG = Double
 
 data Environment n = Environment
   { _style :: Style V2 n
-  , __pre :: T.Text
+  , __pre  :: T.Text
   }
 
 makeLenses ''Environment
@@ -270,14 +271,31 @@ instance SVGFloat n => Backend SVG V2 n where
 
 rtree :: SVGFloat n => RTree SVG V2 n Annotation -> Render SVG V2 n
 rtree (Node n rs) = case n of
-  RPrim p                 -> render SVG p
-  RStyle sty              -> R $ local (over style (<> sty)) r
-  RAnnot (OpacityGroup o) -> R $ g_ [Opacity_ <<- toText o] <$> r
-  RAnnot (Href uri)       -> R $ a_ [XlinkHref_ <<- T.pack uri] <$> r
-  _                       -> R r
+  RPrim p                       -> render SVG p
+  RStyle sty                    -> R $ local (over style (<> sty)) r
+  RAnnot (OpacityGroup o)       -> R $ g_ [Opacity_ <<- toText o] <$> r
+  RAnnot (Href uri)             -> R $ a_ [XlinkHref_ <<- T.pack uri] <$> r
+  RAnnot (KeyVal ("class",v))   -> R $ with <$> r <*> pure [Class_ <<- T.pack v]
+  RAnnot (KeyVal ("id",v))      -> R $ with <$> r <*> pure [Id_ <<- T.pack v]
+  RAnnot (KeyVal ("title",v))   -> R $ do
+    e <- r
+    pure $ g_ [] $ e <> title_ [] (toElement v)
+  _                             -> R r
   where
     R r = foldMap rtree rs
 
+-- | Set the id for a particular SVG diagram
+svgId :: SVGFloat n => String -> QDiagram SVG V2 n Any -> QDiagram SVG V2 n Any
+svgId = curry keyVal "id"
+
+-- | Set the class for a particular SVG diagram
+svgClass :: SVGFloat n => String -> QDiagram SVG V2 n Any -> QDiagram SVG V2 n Any
+svgClass = curry keyVal "class"
+
+-- | Set the title text for a particular SVG diagram
+svgTitle :: SVGFloat n => String -> QDiagram SVG V2 n Any -> QDiagram SVG V2 n Any
+svgTitle = curry keyVal "title"
+
 -- | Lens onto the size of the svg options.
 sizeSpec :: Lens' (Options SVG V2 n) (SizeSpec V2 n)
 sizeSpec f opts = f (_size opts) <&> \s -> opts { _size = s }
@@ -397,7 +415,7 @@ instance SVGFloat n => Renderable (DImage n (Native Img)) SVG where
     mime <- case t of
           'J' -> return "image/jpeg"
           'P' -> return "image/png"
-          _   -> fail   "Unknown mime type while rendering image"
+          _   -> error  "Unknown mime type while rendering image"
     return $ R.renderDImage di $ R.dataUri mime d
 
 instance Hashable n => Hashable (Options SVG V2 n) where
